package com.ibm.commerce.multipleapprovals.commands;

/*******************************************************************************************
//*-----------------------------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*
//*----------------------------------------------------------------------------------------
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its reliability, 
//* serviceability or functionality.  
//*
//* This sample may include the names of individuals, companies, brands and products 
//* in order to illustrate concepts as completely as possible.  All of these names
//* are fictitious and any similarity to the names and addresses used by actual persons 
//* or business enterprises is entirely coincidental.
//*----------------------------------------------------------------------------------------*/


import javax.naming.*;
import java.rmi.*;
import javax.ejb.*;
import com.ibm.commerce.ubf.util.TransitionSortingAttribute;
import com.ibm.commerce.ubf.util.BusinessFlowConstants;
import com.ibm.commerce.ubf.objects.*;
import com.ibm.commerce.approval.objects.*;
import com.ibm.commerce.user.objects.*;
import com.ibm.commerce.membergroup.commands.*;
import java.util.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.approval.util.*;
import com.ibm.commerce.base.objects.*;
import com.ibm.commerce.webcontroller.*;
import com.ibm.commerce.datatype.*;
import com.ibm.commerce.ubf.registry.*;
import com.ibm.commerce.approval.commands.*;

/* This command will create one APRVSTATUS record for each potential approver
   of the action.
*/

public class ApprovalMultipleSetUpRecordsCmdImpl extends ApprovalSetUpRecordsCmdImpl implements ApprovalSetUpRecordsCmd {
	public int multipleApprovalConstant;
	/* ECMessage, if not enough approvers are assigned to the request*/
	
	public ECMessage notEnoughApproversEx;

/**
 * ApprovalMultipleSetUpRecordsCmdImpl constructor comment.
 */
public ApprovalMultipleSetUpRecordsCmdImpl() {
	super();
}
	public java.lang.String getEventIdentifier() {
	return eventIdentifier;
}
	public boolean isApprovalNeeded() {
	return eventIdentifier == null;
}
public boolean isReadyToCallExecute()
{
	// need to have the approver group name to do the set up and at least
	// an org id or user to climb up the chain of
	//
	// also need all the business flow data
	return approverGroupName != null &&
	       (approverOrgId != null || userIdFromApproverOrg != null)  &&
	       businessEntityId != null && businessFlowId != null &&
	       businessFlowTypeId != null && businessFlowStateId != null;
}
public void performExecute() throws ECException {
    final String methodName = "performExecute";
    ECTrace.entry(
        ECTraceIdentifiers.COMPONENT_APPROVAL,
        getClass().getName(),
        methodName);
    boolean foundLowestOrgInChain = false;
    Long[] orgChain;
    Long orgId;
    OrganizationAccessBean org = null;
    java.sql.Timestamp createTime = new java.sql.Timestamp(System.currentTimeMillis());
        
    try {
        // get org chain from either incoming orgid or userid
        if (approverOrgId == null) {
            UserAccessBean user = new UserAccessBean();
            user.setInitKey_MemberId(
                WCSStringConverter.LongToString(userIdFromApproverOrg));
            orgChain = user.getAncestors();
            org = null;
        } else {
            // NOTE: in this case the org itself is not included in the chain (hence the special -1 processing below)
            org = new OrganizationAccessBean();
            org.setInitKey_MemberId(WCSStringConverter.LongToString(approverOrgId));
            orgChain = org.getAncestors();
        }

        // go through the org chain until we find one that has an approval group
        MemberGroupAccessBean group;
        for (int i = -1; i < orgChain.length && !foundLowestOrgInChain; i++) {
            group = null;

            // if first time and there's an org use that this pass.
            if (i == -1) {
                if (org == null) {
                    if (orgChain.length > 0) {
                        i = 0; // move to first one
                        orgId = orgChain[0];
                    } else {
                        orgId = null;
                    }
                } else {
                    orgId = org.getMemberIdInEJBType();
                }
            } else {
                orgId = orgChain[i];
            }

            if (orgId != null) {
                try {
                    group = new MemberGroupAccessBean().findByOwnerName(orgId, approverGroupName);
                } catch (javax.ejb.FinderException e) {
                    // if not found just leave group as null and continue
                    group = null;
                }
            } else {
                group = null;
            }

            if (group != null) {
                foundLowestOrgInChain = true;

                // get the list of approvers (users in this org's approval group)
                ListUsersInMemberGroupCmd listUsersInMemberGroupCmd =
                    (ListUsersInMemberGroupCmd) CommandFactory.createCommand(
                        ListUsersInMemberGroupCmd.Name,
                        getStoreId());
                listUsersInMemberGroupCmd.setCommandContext(getCommandContext());
                listUsersInMemberGroupCmd.setMemberGroupName(approverGroupName);
                listUsersInMemberGroupCmd.setMemberGroupOwnerId(orgId);
                listUsersInMemberGroupCmd.execute();
                UserAccessBean[] approvers = listUsersInMemberGroupCmd.getUsers();

                // if there are no approvers in the approver gorup, throw exception as no one will be able to approve
                if (approvers.length == 0)
                    throw new ECSystemException(
                        ECMessage._ERR_NO_APPROVERS_IN_GROUP,
                        getClass().getName(),
                        methodName,
                        ECMessageHelper.generateMsgParms(approverGroupName, orgId));

                
                // New check for Multiple Approval Flow: If there are not enough approvers 
                // (as specified by multipleApprovalConstant) in the approver group, throw exception
                
                notEnoughApproversEx = new ECMessage(ECMessageSeverity.ERROR,ECMessageType.USER,"_ERR_NOT_ENOUGH_APPROVERS","MultipleApprovals");
                Locale locale = null;
                locale = Locale.getDefault();
                ResourceBundle bundle = ResourceBundle.getBundle("MultipleApprovals", locale);
                multipleApprovalConstant = (int)Integer.parseInt((String) bundle.getObject ("HandleApprovals.multipleApprovalContant"));
            
                CommandContext tmpCmdContext = getCommandContext();
                HttpControllerRequestObject aRequest = (HttpControllerRequestObject)tmpCmdContext.getRequest();                               

                if (approvers.length < multipleApprovalConstant && aRequest.getRequestName().equalsIgnoreCase(ApprovalConstants.EC_APPROVAL_FLOWTYPE_USER_REGISTRATION))
                    throw new ECSystemException(
                        notEnoughApproversEx,
                        getClass().getName(),
                        methodName,
                        ECMessageHelper.generateMsgParms(approverGroupName, orgId));
                
                
                // add an approval record for each approver
                ApprovalStatusAccessBean anApprovalStatusAccessBean = null;
                for (int j = 0; j < approvers.length; j++) {
                    anApprovalStatusAccessBean =
                        new ApprovalStatusAccessBean(
                            businessFlowTypeId,
                            businessFlowId,
                            businessEntityId,
                            businessFlowStateId,
                            commandContext.getUserId(),
                            WCSStringConverter.StringToLong(approvers[j].getUserId()),
                            group.getMbrGrpIdInEJBType(),
                            ApprovalConstants.EC_APPROVAL_STATUS_PENDING,
                            createTime,
                            null);
                    anApprovalStatusAccessBean.commitCopyHelper();
                }
            }
        }

        // if no org in chain has group, do auto-approval
        if (!foundLowestOrgInChain) {
            // Retrieve the Transition record that is marked as an approval
            BusinessFlowManager businessFlowManager = BusinessFlowManager.getInstance();

            BusinessFlowTransition tran =
                businessFlowManager.getTransitionsByFlowIdAndSourceStateIdAndApproval(
                    businessFlowId,
                    businessFlowStateId,
                    com.ibm.commerce.ubf.util.BusinessFlowConstants.EC_APPROVAL)[0];
            eventIdentifier = tran.getEventIdentifier();            
        }

    } catch (CreateException e) {
        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            getClass().getName(),
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e);
    } catch (FinderException e) {
        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            getClass().getName(),
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e);
    } catch (RemoteException e) {
        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            getClass().getName(),
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e);
    } catch (NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_NAMING_EXCEPTION,
            getClass().getName(),
            methodName,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e);
    } catch (ECApplicationException e) {
        throw (ECApplicationException) e;
    } catch (ECSystemException e) {
        throw (ECSystemException) e;
    } catch (Exception e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            getClass().getName(),
            methodName,
            e);
    }

    ECTrace.exit(
        ECTraceIdentifiers.COMPONENT_APPROVAL,
        getClass().getName(),
        methodName);
}
}
