package com.ibm.commerce.multipleapprovals.commands;

/*******************************************************************************************
//*-----------------------------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*
//*----------------------------------------------------------------------------------------
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its reliability, 
//* serviceability or functionality.  
//*
//* This sample may include the names of individuals, companies, brands and products 
//* in order to illustrate concepts as completely as possible.  All of these names
//* are fictitious and any similarity to the names and addresses used by actual persons 
//* or business enterprises is entirely coincidental.
//*----------------------------------------------------------------------------------------*/


import java.util.*;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;

import com.ibm.commerce.tools.command.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.datatype.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.ubf.event.*;
import com.ibm.commerce.ubf.util.*;
import com.ibm.commerce.approval.util.*;
import com.ibm.commerce.approval.objects.*;
import com.ibm.commerce.tools.common.ui.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.ubf.registry.*;
import com.ibm.commerce.base.util.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.approval.commands.*;

/**
 Processes approvals or rejections. The approver may either approve or
 reject one or more approval requests.  The approver may supply an optional
 comment which will be stored in the approval request's record in the APRVSTATUS
 table.  This comment can be viewed by the request submitter.
 <P><strong>Behaviour</strong>
 <P>This command will handle the updating of the APRVSTATUS table as well as
 any other activities. The status of the approval requests record in the APRVSTATUS 
 table will be updated to 1 for approved or 2 for rejected.
 <P>The command treats each approval or rejection as a separate transaction so it is
 possible that if command is invoked to process a batch of approvals or rejections,
 some may succeed and others may fail.
 Upon the last required approval or rejection super.performExecute() will be called
 raising a business flow event to handle each approval or rejection.
 The business flow event will handle the updating of the APRVSTATUS table as well as
 any other activities which have been defined for this transition 
 <P>The command will set the resultmsg for the view it sets to indicate the success
 or failure of the processing.  The value of resultmsg will be the appropriate key
 into the approvalsNLS properties file for the message to be displayed by the JSP.
**/

public class HandleMultipleApprovalsCmdImpl extends HandleApprovalsCmdImpl implements HandleApprovalsCmd {

	 public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	 private final String _STR_CLASSNAME = this.getClass().getName();
	 protected String aprvIds = null;
	 protected Integer aprvAct = null;
	 protected String viewTask = null;
	 private Long idsArray[] = null;
	 protected String comments = null;
	 protected Long submitterId = null;
	 protected com.ibm.commerce.base.util.SortingAttribute  sortAttrib;
	 protected java.util.Enumeration aprvStatusBySubmitter;
	 protected int multipleApprovalConstant = 0;	
	 
public TypedProperty getViewInputProperties()
{
		super.getViewInputProperties();
		viewReqProperties.remove(ApprovalConstants.EC_PARM_APRV_ACT);
		viewReqProperties.remove(ApprovalConstants.EC_PARM_APRV_IDS);
		viewReqProperties.remove(ApprovalConstants.EC_PARM_VIEWTASK);
		viewReqProperties.remove(ApprovalConstants.EC_PARM_APRV_COMMENTS);
		viewReqProperties.remove(BusinessFlowConstants.EC_BUSINESS_FLOW_EVENT_IDENTIFIER);
	    viewReqProperties.remove(BusinessFlowConstants.EC_FLOWTYPE_IDENTIFIER);
	    viewReqProperties.remove(BusinessFlowConstants.EC_ENTITY_ID);
	    return viewReqProperties;
}
	 public String getViewTask()
{
	return viewTask;
}
public void performExecute() throws ECSystemException
{
   String method_name = "performExecute";
   		
   ECTrace.entry(ECTraceIdentifiers.COMPONENT_APPROVAL, _STR_CLASSNAME, method_name);
	
   String eventIdentifier = null;
   String flowTypeIdentifier = null;
   Long entityId = null;
   Long flowId = null;
   Long stateId = null;
   Long flowTypeId = null;
   ApprovalStatusAccessBean theApprovalStatusBean;
   ApprovalStatusAccessBean anApprovalStatusBean;   
   String resultMsg = null;
   String failMsg = null;
   String successMsg = null;
   String partialMsg = null;
   int successes = 0;
   int failures = 0;
   int aprvFlag = aprvAct.intValue();   
   Long tmpStatus;
   int statusFlag;
   boolean jmpForLoop;
   boolean handleApprovalsCmdFlag;
   ApprovalStatusAccessBean chkApprovalStatusBean;
      
   // Set the various result messages based on the flag.
   // The EC_RESULTMSG_OK, EC_RESULTMSG_FAILED and EC_RESULTMSG_PARTIAL arrays
   // in ApprovalConstants should have length equal to (1 + the maximum value
   // of aprvAct).  If this is not the case, the default value (the first
   // element) will be used.
   if(aprvFlag < 0 || aprvFlag >= ApprovalConstants.EC_APPROVAL_CODE_GUI_ARRAY_SIZES)
   {
	   successMsg = ApprovalConstants.EC_RESULTMSG_OK[0];
	   failMsg = ApprovalConstants.EC_RESULTMSG_FAILED[0];
	   partialMsg = ApprovalConstants.EC_RESULTMSG_PARTIAL[0];
   }
   else
   {
	   successMsg = ApprovalConstants.EC_RESULTMSG_OK[aprvFlag];
	   failMsg = ApprovalConstants.EC_RESULTMSG_FAILED[aprvFlag];
	   partialMsg = ApprovalConstants.EC_RESULTMSG_PARTIAL[aprvFlag];
   }
   
   // Will be used by the HandleApprovalsCmdImpl Command
   responseProperties = new TypedProperty();   
   
   // To get the flowType. Multiple Approvals is activated only for 
   // the 'UserRegistrationAdd' Business Flow process.
   BusinessFlowManager businessFlowManager = BusinessFlowManager.getInstance();
   BusinessFlowTransition[] transitions;
   BusinessFlowType theFlowType;
   
   // Loop for number of approvals to be processed
   for(int i = 0; i < idsArray.length; i++)
   {	   
	   handleApprovalsCmdFlag = false;
	   jmpForLoop = false;
	   try
	   {		   
		   theApprovalStatusBean = new ApprovalStatusAccessBean();
		   try
		   {
			   theApprovalStatusBean = theApprovalStatusBean.findByIdForUpdate(idsArray[i]);
		   }
		   catch(Exception e)
		   {
			   jmpForLoop = true;
		   }
		   
		   if (!jmpForLoop)
		   {
			   // Retreive Approval Status (APRVSTATUS) record			   
			   entityId = Long.valueOf(theApprovalStatusBean.getEntityId());
			   flowId = Long.valueOf(theApprovalStatusBean.getFlowId());
			   flowTypeId = Long.valueOf(theApprovalStatusBean.getFlowTypeId());
			   stateId = Long.valueOf(theApprovalStatusBean.getStateId());
			   submitterId = Long.valueOf(theApprovalStatusBean.getSubmitterId());
			   aprvStatusBySubmitter = theApprovalStatusBean.findBySubmitter(submitterId, sortAttrib);
			   theFlowType = businessFlowManager.getFlowType(flowTypeId);
			   flowTypeIdentifier = theFlowType.getIdentifier();
			   
			   // If the action 'reject' or the Business Flow is not 'UserRegistrationAdd'; Call HandleApprovalsCmd
			   // command. Note: To enable Multiple Approval for other business processes, alter conditions for this 'if statement'.
			   if (aprvFlag == 2 || !flowTypeIdentifier.equalsIgnoreCase(ApprovalConstants.EC_APPROVAL_FLOWTYPE_USER_REGISTRATION)) 
			   {
				   handleApprovalsCmdFlag = true;
				   try {
					   requestProperties.remove("aprv_ids");
					   requestProperties.put("aprv_ids",idsArray[i]); //Pass in only current Approval record ID to the super class
					   super.setRequestProperties(requestProperties);
					   super.performExecute();
				   }
				   catch(Exception e) 
				   {
					   failures++;
					   throw new ECSystemException(ECMessage._ERR_GENERIC, _STR_CLASSNAME, method_name, new Object[] { e.toString() }, e);
				   }
			   }
			   
			   if (!handleApprovalsCmdFlag)
			   {
				   Vector multipleStatus = new Vector();
				   tmpStatus = null;
				   statusFlag = 0;
				   
				   while (aprvStatusBySubmitter.hasMoreElements())
				   {
					   anApprovalStatusBean = new ApprovalStatusAccessBean();
					   anApprovalStatusBean = (ApprovalStatusAccessBean) aprvStatusBySubmitter.nextElement();
					   tmpStatus = (Long.valueOf(anApprovalStatusBean.getStatus()));
					   if (tmpStatus.intValue() == ApprovalConstants.EC_STATUS_APPROVED)
					   statusFlag++;
				   }
				   if (statusFlag < (multipleApprovalConstant - 1)) 
				   {
					   theApprovalStatusBean.setStatus( new Integer(ApprovalConstants.EC_STATUS_APPROVED));
					   theApprovalStatusBean.commitCopyHelper();
				   }
				   else 
				   {
					   requestProperties.remove("aprv_ids");
					   requestProperties.put("aprv_ids",idsArray[i]);
					   super.setRequestProperties(requestProperties);
					   super.performExecute();
				   }
			   }
		   }
	   }// end of try { ......
	   
	   catch(CreateException e) { throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, _STR_CLASSNAME, method_name, new Object[] { e.toString() }, e); }
	   catch(FinderException e) { throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, _STR_CLASSNAME, method_name, new Object[] { e.toString() }, e); }		   
	   catch(RemoteException e) { throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, _STR_CLASSNAME, method_name, new Object[] { e.toString() }, e); }
	   catch(NamingException e) { throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, _STR_CLASSNAME, method_name, new Object[] { e.toString() }, e); }
	   catch(Exception e)       { throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, _STR_CLASSNAME, method_name, new Object[] { e.toString() }, e); }
   } // end for (int i = 0.......   
   
   // Determine what message to return to the view
   if (failures > 0)
       resultMsg = failMsg;
   else
   	   resultMsg = successMsg;
      
   //set response properties
   responseProperties = new TypedProperty();
   responseProperties.put(ApprovalConstants.EC_RESULTMSG,resultMsg);
   responseProperties.put(ECConstants.EC_VIEWTASKNAME,ECConstants.EC_GENERIC_REDIRECTVIEW);
   responseProperties.put(ECConstants.EC_REDIRECTURL,getViewTask());
   ECTrace.exit(ECTraceIdentifiers.COMPONENT_APPROVAL, _STR_CLASSNAME, method_name);
}
public void reset()
{
  aprvIds = null;
  aprvAct = null;
  viewTask = null;
  idsArray = null;
  comments = null;
}
public void setRequestProperties(TypedProperty reqParms) throws ECApplicationException 
{
	String method_name = "setRequestProperties";
		
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_APPROVAL, _STR_CLASSNAME, method_name);

	requestProperties = reqParms;

	try 
	{
		// Set the number of approvals required
		Locale locale = null;
     	locale = Locale.getDefault();
     	ResourceBundle bundle = ResourceBundle.getBundle("MultipleApprovals", locale);
     	multipleApprovalConstant = (int ) Integer.parseInt((String) bundle.getObject ("HandleApprovals.multipleApprovalContant"));
		
		// Action must be specified
		aprvAct = reqParms.getInteger(ApprovalConstants.EC_PARM_APRV_ACT);
		
		// A list of approval ids to act on must be specified
		aprvIds = reqParms.getString(ApprovalConstants.EC_PARM_APRV_IDS);
		
	    // View Task name must be specified
	    viewTask = reqParms.getString(ApprovalConstants.EC_PARM_VIEWTASK);
		
        // Split the approval ids string into an array of ids
	    // Make sure that each supplied id is a valid number
	    StringTokenizer list = new StringTokenizer(aprvIds, ",");
	    int i = 0;
	    idsArray = new Long[list.countTokens()];
	    while (list.hasMoreTokens())
	    {
		  idsArray[i] = Long.valueOf(list.nextToken());
	      i++;
	     }

	    // At least one approval id must be supplied   	
	    if(idsArray.length < 1)
    	{
	      throw new ECApplicationException(ECMessage._ERR_CMD_INVALID_PARAM, _STR_CLASSNAME, method_name);
    	}    

    	// comments is an optional parameter
        comments = reqParms.getString(ApprovalConstants.EC_PARM_APRV_COMMENTS,null);
	    	     
    	if(comments != null)
     	{
	       // Make sure the comment length does not exceed the limit
	       if(comments.length() > 254)
	       {
	    	 throw new ECApplicationException(ECMessage._ERR_CMD_INVALID_PARAM, _STR_CLASSNAME, method_name);
	       }
	    }
     	
     	
    } 
	catch (Exception e) 
	{
		throw new ECApplicationException(ECMessage._ERR_CMD_MISSING_PARAM, _STR_CLASSNAME, method_name);
	}
	
}
}
